package icu.fur93.esp32_car

import android.Manifest
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothGatt
import android.bluetooth.BluetoothGattCallback
import android.bluetooth.BluetoothGattCharacteristic
import android.bluetooth.BluetoothGattDescriptor
import android.bluetooth.le.BluetoothLeScanner
import android.bluetooth.le.ScanCallback
import android.bluetooth.le.ScanFilter
import android.bluetooth.le.ScanResult
import android.bluetooth.le.ScanSettings
import android.content.pm.ActivityInfo
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.os.ParcelUuid
import android.util.Log
import android.view.WindowManager
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.graphicsLayer
import androidx.compose.ui.graphics.toArgb
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.unit.dp
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.core.view.WindowCompat
import androidx.core.view.WindowInsetsCompat
import androidx.core.view.WindowInsetsControllerCompat
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import icu.fur93.esp32_car.page.ControlGamepadModePage
import icu.fur93.esp32_car.page.ControlPage
import icu.fur93.esp32_car.page.ControlPathfinderModePage
import icu.fur93.esp32_car.page.ControlSingleJoystickModePage
import icu.fur93.esp32_car.page.GamepadScreen
import icu.fur93.esp32_car.page.HomePage
import icu.fur93.esp32_car.page.SettingsPage
import icu.fur93.esp32_car.ui.component.Joystick
import icu.fur93.esp32_car.ui.component.JoystickState
import icu.fur93.esp32_car.ui.theme.Esp32carTheme
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import java.util.UUID
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale

class MainActivity : ComponentActivity() {
    private val serviceUUID = "6E400001-B5A3-F393-E0A9-E50E24DCCA9E"
    private val rxCharUUID = "6E400002-B5A3-F393-E0A9-E50E24DCCA9E"
    private val txCharUUID = "6E400003-B5A3-F393-E0A9-E50E24DCCA9E"

    private var bleScanner: BluetoothLeScanner? = null

    // 将deviceList移动到Activity作用域内并使用MutableList
    private val deviceList = mutableListOf<ScanResult>()

    // 添加可观察的连接状态
    private val _isConnected = mutableStateOf(false)
    val isConnected: State<Boolean> = _isConnected

    private val scanCallback = object : ScanCallback() {
        override fun onScanResult(callbackType: Int, result: ScanResult) {
            super.onScanResult(callbackType, result)
            Log.d("MainActivity", "onScanResult: ${result.device.name} - ${result.device.address}")
            if (!deviceList.any { it.device.address == result.device.address }) {
                deviceList.add(result)
            }
        }

        override fun onScanFailed(errorCode: Int) {
            super.onScanFailed(errorCode)
            Log.e("MainActivity", "扫描失败，错误码: $errorCode")
        }
    }

    private val carController by lazy {
        CarController { }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        // 设置全屏显示
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
            window.attributes.layoutInDisplayCutoutMode =
                WindowManager.LayoutParams.LAYOUT_IN_DISPLAY_CUTOUT_MODE_SHORT_EDGES
        }

//        window.statusBarColor = ContextCompat
//
//        // 隐藏状态栏和导航栏
//        WindowInsetsControllerCompat(window, window.decorView).let { controller ->
//            controller.hide(WindowInsetsCompat.Type.systemBars())
//            controller.systemBarsBehavior =
//                WindowInsetsControllerCompat.BEHAVIOR_SHOW_TRANSIENT_BARS_BY_SWIPE
//        }
//
//        // 设置横屏
//        requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE

        // 检查并请求所有必要的权限
        val requiredPermissions = arrayOf(
            Manifest.permission.BLUETOOTH_SCAN,
            Manifest.permission.BLUETOOTH_CONNECT,
            Manifest.permission.ACCESS_FINE_LOCATION,
            Manifest.permission.ACCESS_COARSE_LOCATION
        )

        val missingPermissions = requiredPermissions.filter {
            ActivityCompat.checkSelfPermission(this, it) != PackageManager.PERMISSION_GRANTED
        }

        if (missingPermissions.isNotEmpty()) {
            ActivityCompat.requestPermissions(
                this,
                missingPermissions.toTypedArray(),
                BLUETOOTH_PERMISSION_REQUEST_CODE
            )
        } else {
            // 如果已经有所有权限，直接开始扫描
            val bluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
            bleScanner = bluetoothAdapter?.bluetoothLeScanner
            startBleScan()
        }

        setContent {
            Esp32carTheme {
//                GamepadScreen(
//                    deviceList = deviceList,
//                    isConnected = isConnected.value,
//                    onStartScan = ::startBleScan,
//                    onConnect = ::connectToDevice,
//                    carController = carController
//                )
                App()
            }
        }
    }

    private fun connectToDevice(result: ScanResult) {
        if (ActivityCompat.checkSelfPermission(
                this,
                Manifest.permission.BLUETOOTH_CONNECT
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(
                this,
                arrayOf(Manifest.permission.BLUETOOTH_CONNECT),
                BLUETOOTH_PERMISSION_REQUEST_CODE
            )
            return
        }

        result.device.connectGatt(this, false, object : BluetoothGattCallback() {
            override fun onConnectionStateChange(gatt: BluetoothGatt?, status: Int, newState: Int) {
                super.onConnectionStateChange(gatt, status, newState)
                when (newState) {
                    BluetoothGatt.STATE_CONNECTED -> {
                        carController.bluetoothGatt = gatt
                        _isConnected.value = true
                        gatt?.discoverServices()
                    }

                    BluetoothGatt.STATE_DISCONNECTED -> {
                        _isConnected.value = false
                        carController.bluetoothGatt = null
                    }
                }
            }

            override fun onServicesDiscovered(gatt: BluetoothGatt?, status: Int) {
                super.onServicesDiscovered(gatt, status)
                if (status == BluetoothGatt.GATT_SUCCESS) {
                    gatt?.getService(UUID.fromString(serviceUUID))?.let { service ->
                        carController.bluetoothGatt = gatt
                        carController.rxCharacteristic =
                            service.getCharacteristic(UUID.fromString(rxCharUUID))

                        // 注册通知监听器
                        val txCharacteristic =
                            service.getCharacteristic(UUID.fromString(txCharUUID))
                        gatt.setCharacteristicNotification(txCharacteristic, true)
                        txCharacteristic.descriptors.forEach { descriptor ->
                            descriptor.value = BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE
                            gatt.writeDescriptor(descriptor)
                        }
                    }
                }
            }

            override fun onCharacteristicChanged(
                gatt: BluetoothGatt,
                characteristic: BluetoothGattCharacteristic,
                value: ByteArray
            ) {
                super.onCharacteristicChanged(gatt, characteristic, value)
                if (characteristic.uuid == UUID.fromString(txCharUUID)) {
                    carController.onReceivePacket(value)
                }
            }
        })
    }

    private fun startBleScan() {
        if (ActivityCompat.checkSelfPermission(
                this,
                Manifest.permission.BLUETOOTH_SCAN
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(
                this,
                arrayOf(
                    Manifest.permission.BLUETOOTH_SCAN,
                    Manifest.permission.ACCESS_FINE_LOCATION
                ),
                BLUETOOTH_PERMISSION_REQUEST_CODE
            )
            return
        }

        // 先停止当前扫描
        try {
            bleScanner?.stopScan(scanCallback)
        } catch (e: Exception) {
            Log.e("MainActivity", "停止扫描失败: ${e.message}")
        }

        // 清空之前的扫描结果
        deviceList.clear()

        Log.d("MainActivity", "开始扫描...")

        // 添加UUID过滤器
        val scanFilter = ScanFilter.Builder()
            .setServiceUuid(ParcelUuid.fromString(serviceUUID))
            .build()

        val scanSettings = ScanSettings.Builder()
            .setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY)
            .build()

        try {
//            bleScanner?.startScan(listOf(scanFilter), scanSettings, scanCallback)
            bleScanner?.startScan(scanCallback)
        } catch (e: Exception) {
            Log.e("MainActivity", "开始扫描失败: ${e.message}")
        }

        // 10秒后停止扫描
        Handler(Looper.getMainLooper()).postDelayed({
            stopBleScan()
            Log.d("MainActivity", "停止扫描，发现 ${deviceList.size} 个设备")
        }, 10000)
    }

    private fun stopBleScan() {
        if (ActivityCompat.checkSelfPermission(
                this,
                Manifest.permission.BLUETOOTH_SCAN
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            return
        }
        try {
            bleScanner?.stopScan(scanCallback)
        } catch (e: Exception) {
            Log.e("MainActivity", "停止扫描失败: ${e.message}")
        }
    }

    // 添加权限请求结果处理
    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == BLUETOOTH_PERMISSION_REQUEST_CODE) {
            if (grantResults.all { it == PackageManager.PERMISSION_GRANTED }) {
                Log.d("MainActivity", "所有权限已获得")
                // 权限获得后重新初始化蓝牙
                val bluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
                bleScanner = bluetoothAdapter?.bluetoothLeScanner
            } else {
                Log.e("MainActivity", "部分权限未获得")
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        // 确保在Activity销毁时停止扫描
        stopBleScan()
    }

    companion object {
        private const val BLUETOOTH_PERMISSION_REQUEST_CODE = 1
    }
}

@Composable
fun App() {
    val activity = LocalContext.current as MainActivity
    activity.window.statusBarColor = MaterialTheme.colorScheme.background.toArgb()

    val navController = rememberNavController()

    NavHost(
        navController = navController,
        startDestination = Route.Home.route
    ) {
        composable(Route.Home.route) { HomePage(navController) }
        composable(Route.Control.route) { ControlPage(navController) }
        composable(Route.Settings.route) { SettingsPage(navController) }
        composable(Route.ControlPathfinderMode.route) { ControlPathfinderModePage(navController) }
        composable(Route.ControlSingleJoystickMode.route) { ControlSingleJoystickModePage(navController) }
        composable(Route.ControlGamepadMode.route) { ControlGamepadModePage(navController) }
    }
}