package icu.fur93.esp32_car.page

import android.bluetooth.le.ScanResult
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Button
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Slider
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.graphicsLayer
import androidx.compose.ui.unit.dp
import icu.fur93.esp32_car.CarController
import icu.fur93.esp32_car.LogDirection
import icu.fur93.esp32_car.ui.component.Joystick
import icu.fur93.esp32_car.ui.component.JoystickState
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale


@Composable
fun ControlJoystick(
    modifier: Modifier,
    carController: CarController,
    joystickState: JoystickState,
    onJoystickStateChange: (JoystickState) -> Unit
) {
    // 记住上一次发送的状态和时间
    val lastSentState = remember { mutableStateOf(JoystickState()) }
    val lastSentTime = remember { mutableStateOf(0L) }

    // 用于停止命令重发的计数器
    val stopCommandCount = remember { mutableStateOf(0) }
    val coroutineScope = rememberCoroutineScope()

    Joystick(
        modifier = modifier,
        size = 350f
    ) { state ->
        // 更新状态
        onJoystickStateChange(state)

        val currentTime = System.currentTimeMillis()

        // 检查是否是停止状态（摇杆回正）
        val isStopCommand = state.x == 0f && state.y == 0f

        // 如果是停止命令，启动重发机制
        // 否则检查时间间隔和状态变化
        if (isStopCommand) {
            if (stopCommandCount.value == 0) {
                // 第一次发送停止命令
                carController.stop()
                stopCommandCount.value = 1

                // 启动延时重发
                coroutineScope.launch {
                    delay(50) // 延时50ms
                    carController.stop() // 再次发送停止命令
                    stopCommandCount.value = 2

                    delay(50) // 再次延时50ms
                    carController.stop() // 第三次发送停止命令
                    stopCommandCount.value = 0 // 重置计数器
                }
            }
        } else if ((state.x != lastSentState.value.x || state.y != lastSentState.value.y) &&
            currentTime - lastSentTime.value >= 50) {
            // 非停止命令的正常处理
            stopCommandCount.value = 0 // 重置停止命令计数器
            val x = (state.x * 100).toInt()
            val y = (state.y * 100).toInt()
            carController.sendXYR(x, y, 0)

            // 更新上次发送的状态和时间
            lastSentState.value = state
            lastSentTime.value = currentTime
        }
    }
}

@Composable
fun GamepadScreen(
    deviceList: List<ScanResult>,
    isConnected: Boolean,
    onStartScan: () -> Unit,
    onConnect: (ScanResult) -> Unit,
    carController: CarController
) {
    var showDeviceList by remember { mutableStateOf(false) }
    var sliderValue by remember { mutableStateOf(0f) }
    var joystickState by remember { mutableStateOf(JoystickState()) }

    Box(modifier = Modifier.fillMaxSize()) {
        Column(modifier = Modifier.fillMaxWidth()) {
            // 电机状态显示区域
            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(8.dp),
                horizontalArrangement = Arrangement.SpaceEvenly
            ) {
                // 摇杆状态显示
                Column {
                    Text("摇杆")
                    Text("X: %.2f".format(joystickState.x))
                    Text("Y: %.2f".format(joystickState.y))
                }

                // 电机A状态
                Column {
                    Text("电机A")
                    Text("PWM: ${carController.carState.value.motorAState.pwm}")
                    Text("IN1: ${carController.carState.value.motorAState.in1}")
                    Text("IN2: ${carController.carState.value.motorAState.in2}")
                }

                // 电机B状态
                Column {
                    Text("电机B")
                    Text("PWM: ${carController.carState.value.motorBState.pwm}")
                    Text("IN1: ${carController.carState.value.motorBState.in1}")
                    Text("IN2: ${carController.carState.value.motorBState.in2}")
                }

                // 电机C状态
                Column {
                    Text("电机C")
                    Text("PWM: ${carController.carState.value.motorCState.pwm}")
                    Text("IN1: ${carController.carState.value.motorCState.in1}")
                    Text("IN2: ${carController.carState.value.motorCState.in2}")
                }

                // 电机D状态
                Column {
                    Text("电机D")
                    Text("PWM: ${carController.carState.value.motorDState.pwm}")
                    Text("IN1: ${carController.carState.value.motorDState.in1}")
                    Text("IN2: ${carController.carState.value.motorDState.in2}")
                }

                // 设备选择按钮
                Button(
                    onClick = { showDeviceList = true },
                ) {
                    Text(if (isConnected) "已连接" else "选择设备")
                }
            }

            // 方向控制布局
            Row(
                modifier = Modifier.fillMaxSize(),
                horizontalArrangement = Arrangement.SpaceEvenly,
                verticalAlignment = Alignment.CenterVertically
            ) {
                // 左侧控制区
                Column(
                    horizontalAlignment = Alignment.CenterHorizontally,
                    verticalArrangement = Arrangement.Center,
                    modifier = Modifier
                        .weight(1f)
                        .fillMaxHeight()
                ) {
                    Row(
                        horizontalArrangement = Arrangement.Center,
                        verticalAlignment = Alignment.CenterVertically,
                        modifier = Modifier
                            .fillMaxWidth()
                            .padding(vertical = 16.dp)
                    ) {
                        ControlJoystick(
                            modifier = Modifier.padding(16.dp),
                            carController = carController,
                            joystickState = joystickState,
                            onJoystickStateChange = { newState ->
                                joystickState = newState
                            }
                        )
                    }
                }

                // 中间 Log 区
                // 用于显示发出的蓝牙指令
                LazyColumn(
                    modifier = Modifier
                        .fillMaxSize()
                        .padding(4.dp)
                        .weight(1f),
                    reverseLayout = true // 反转布局顺序
                ) {
                    items(carController.logs.value.filter { it.direction == LogDirection.SEND }.reversed()) { log ->
                        Row(
                            modifier = Modifier
                                .fillMaxWidth()
                                .padding(vertical = 2.dp),
                            horizontalArrangement = Arrangement.SpaceBetween
                        ) {
                            Text(
                                text = "[${
                                    SimpleDateFormat("HH:mm:ss.SSS", Locale.getDefault())
                                    .format(Date(log.timestamp))}]",
                                style = MaterialTheme.typography.bodySmall
                            )
                            Text(
                                text = log.data,
                                style = MaterialTheme.typography.bodySmall,
                                modifier = Modifier.weight(1f)
                            )
                        }
                    }
                }

                // 右侧控制区
                Column(
                    horizontalAlignment = Alignment.CenterHorizontally,
                    verticalArrangement = Arrangement.Center,
                    modifier = Modifier.weight(1f)
                ) {
                    // 垂直滑杆
                    Slider(
                        value = sliderValue,
                        onValueChange = { newValue ->
                            // 在0附近添加吸附效果
                            val snapValue = when {
                                newValue > -0.1f && newValue < 0.1f -> 0f
                                else -> newValue
                            }
                            sliderValue = snapValue
                            val speed = (snapValue * 255).toInt()
                            if (speed > 0) {
                                carController.moveForward(speed)
                            } else if (speed < 0) {
                                carController.moveBackward(-speed)
                            } else {
                                carController.stop()
                            }
                        },
                        valueRange = -1f..1f,
                        modifier = Modifier
                            .height(10.dp)
                            .width(200.dp)
                            .padding(vertical = 16.dp)
                            .graphicsLayer(rotationZ = 270f)
                    )
                }
            }
        }

        // 设备列表对话框
        if (showDeviceList) {
            AlertDialog(
                onDismissRequest = { showDeviceList = false },
                title = { Text("选择设备") },
                text = {
                    // 设备列表
                    LazyColumn {
                        items(deviceList) { device ->
                            TextButton(
                                onClick = {
                                    onConnect(device)
                                    showDeviceList = false
                                },
                                modifier = Modifier.fillMaxWidth()
                            ) {
                                Column(
                                    modifier = Modifier.fillMaxWidth(),
                                    horizontalAlignment = Alignment.Start
                                ) {
                                    Text(device.device.name ?: "未知设备")
                                    Text(
                                        device.device.address,
                                        style = MaterialTheme.typography.bodySmall
                                    )
                                }
                            }
                        }
                    }
                },
                dismissButton = {
                    TextButton(onClick = { onStartScan() }) {
                        Text("扫描设备")
                    }
                },
                confirmButton = {
                    TextButton(onClick = { showDeviceList = false }) {
                        Text("取消")
                    }
                }
            )
        }
    }
}